// Generated by CoffeeScript 1.12.6
(function() {
  var _, assert;

  assert = require('cassert');

  _ = require('lodash');

  describe("[pimatic]", function() {
    var config, configFile, env, framework, fs, os;
    framework = null;
    env = null;
    config = {
      settings: {
        debug: true,
        logLevel: "debug",
        httpServer: {
          enabled: true,
          port: 8080
        },
        httpsServer: {},
        database: {
          client: "sqlite3",
          connection: {
            filename: ':memory:'
          }
        }
      },
      plugins: [
        {
          "locations": [
            {
              "name": "office",
              "tag": "Office",
              "ssids": ["OFFICE"],
              "gps": {
                "latitude": 53.5544809,
                "longitude": 9.9786172,
                "radius": 250
              },
              "cids": ["GSM:42407.5455365", "GSM:1401.43266861", "GSM:42407.5455361"]
            }, {
              "name": "home",
              "tag": "Home",
              "ssids": ["HOME"],
              "gps": {
                "latitude": 53.658156,
                "longitude": 10.087347,
                "radius": 250
              },
              "cids": ["GSM:42441.5453313", "GSM:1401.43266781", "GSM:411.13021"]
            }, {
              "name": "mall",
              "tag": "AEZ",
              "ssids": [],
              "gps": {
                "latitude": 53.6543899,
                "longitude": 10.0885968,
                "radius": 250
              },
              "cids": []
            }, {
              "name": "train",
              "tag": "S-Bahn",
              "ssids": [],
              "gps": {
                "latitude": 53.652371,
                "longitude": 10.0896482,
                "radius": 250
              },
              "cids": []
            }, {
              "name": "change",
              "tag": "Ohlsdorf",
              "ssids": [],
              "gps": {
                "latitude": 53.6227401,
                "longitude": 10.0329287,
                "radius": 250
              },
              "cids": []
            }
          ],
          "plugin": "phone",
          "active": true,
          "debug": true
        }
      ],
      devices: [
        {
          "id": "phone",
          "name": "phone",
          "class": "PhoneDevice",
          "debug": true
        }
      ],
      rules: [],
      users: [
        {
          username: "admin",
          password: "admin",
          role: "admin"
        }
      ],
      roles: [
        {
          name: "admin",
          permissions: {
            pages: "write",
            rules: "write",
            variables: "write",
            messages: "write",
            events: "write",
            devices: "write",
            groups: "write",
            plugins: "write",
            updates: "write",
            database: "write",
            config: "write",
            controlDevices: true,
            restart: true
          }
        }
      ],
      variables: []
    };
    fs = require('fs');
    os = require('os');
    configFile = (os.tmpdir()) + "/pimatic-test-config.json";
    before(function() {
      var startup;
      fs.writeFileSync(configFile, JSON.stringify(config));
      process.env.PIMATIC_CONFIG = configFile;
      startup = require('./startup');
      env = startup.env;
      return startup.startup().then(function(fw) {
        return framework = fw;
      })["catch"](function(err) {
        return env.logger.error(err);
      });
    });
    after(function() {
      return fs.unlinkSync(configFile);
    });
    describe("[startup]", function() {
      var url;
      url = "http://localhost:" + config.settings.httpServer.port;
      if (config.settings.httpServer.enabled) {
        it("httpServer should run", function(done) {
          var http;
          http = require('http');
          return http.get(url, function(result) {
            return done();
          }).on("error", function(e) {
            throw e;
          });
        });
        return it("httpServer should ask for password", function(done) {
          var http;
          http = require('http');
          return http.get(url, function(result) {
            assert(result.statusCode === 401);
            return done();
          }).on("error", function(e) {
            throw e;
          });
        });
      } else {
        return it("should be initialized", function() {
          return assert(framework != null);
        });
      }
    });
    return describe("[plugin]", function() {
      var pluginConfig;
      pluginConfig = null;
      it('should have locations configuration', function(done) {
        pluginConfig = framework.pluginManager.getPluginConfig('phone');
        assert(pluginConfig != null ? pluginConfig.locations : void 0);
        return done();
      });
      return describe("[phone]", function() {
        var http, phone, phoneCheckLocation, phoneLOC, phoneLocation, phoneRequest, phoneStatus;
        http = require('http');
        phone = null;
        phoneRequest = function(path, callback) {
          var opts, req;
          opts = {
            port: parseInt(config.settings.httpServer.port),
            auth: "admin:admin",
            path: path
          };
          console.log("");
          req = http.get(opts, function(res) {
            var body;
            body = "";
            res.on("data", function(chunk) {
              return body += chunk.toString();
            });
            return res.on("end", function() {
              return callback(body);
            });
          });
          return req.on("error", function(e) {
            throw e;
          });
        };
        phoneStatus = function(deviceId) {
          return console.log("status: id=" + phone.id + " tag=" + phone._tag + " src=" + phone._source + " type=" + phone._type + " lat=" + phone._latitude + " long=" + phone._longitude);
        };
        phoneLocation = function(tag) {
          return _.find(pluginConfig != null ? pluginConfig.locations : void 0, function(location) {
            return location.tag === tag;
          });
        };
        phoneCheckLocation = function(deviceId, tag) {
          return phone._tag === tag && phone._latitude === phoneLocation(tag).gps.latitude && phone._longitude === phoneLocation("Home").gps.longitude;
        };
        phoneLOC = function(tag) {
          var gps;
          gps = phoneLocation(tag).gps;
          return gps.latitude + "," + gps.longitude + "," + gps.radius;
        };
        it("device should be available", function(done) {
          phone = framework.deviceManager.getDeviceById("phone");
          assert(phone != null);
          return done();
        });
        describe("[updateTag]", function() {
          it("should set initial location to \"Home\"", function(done) {
            return phoneRequest("/api/device/phone/updateTag?tag=Home", function(body) {
              assert(JSON.parse(body).success);
              phoneStatus("phone");
              phoneCheckLocation("phone", "Home");
              return done();
            });
          });
          it("should change location tag to \"Office\"", function(done) {
            return phoneRequest("/api/device/phone/updateTag?tag=Office", function(body) {
              assert(JSON.parse(body).success);
              phoneStatus("phone");
              phoneCheckLocation("phone", "Office");
              return done();
            });
          });
          return it("should change location tag to \"Office\"", function(done) {
            return phoneRequest("/api/device/phone/updateTag?tag=Office", function(body) {
              assert(JSON.parse(body).success);
              phoneStatus();
              phoneCheckLocation("Office");
              return done();
            });
          });
        });
        return describe("[updatePhone]", function() {
          var cellid, loc, locn, path, serial, ssid;
          path = "/api/device/phone/updatePhone";
          serial = "0815";
          ssid = "%SSID";
          cellid = "%CELLID";
          locn = "%LOCN";
          loc = "%LOC";
          it("should set location to \"Home\" by %SSID", function(done) {
            var parms;
            ssid = "HOME";
            cellid = "%CELLID";
            locn = "%LOCN";
            loc = "%LOC";
            parms = "?serial=" + serial + "&ssid=" + ssid + "&cellid=" + cellid + "&locn=" + locn + "&loc=" + loc;
            return phoneRequest(path + parms, function(body) {
              assert(JSON.parse(body).success);
              phoneStatus("phone");
              phoneCheckLocation("phone", "Home");
              return done();
            });
          });
          it("should set location to \"Office\" by %LOCN", function(done) {
            var parms;
            ssid = "UNKNOWN";
            cellid = "%CELLID";
            locn = phoneLOC("Office");
            loc = "%LOC";
            parms = "?serial=" + serial + "&ssid=" + ssid + "&cellid=" + cellid + "&locn=" + locn + "&loc=" + loc;
            return phoneRequest(path + parms, function(body) {
              assert(JSON.parse(body).success);
              phoneStatus("phone");
              phoneCheckLocation("phone", "Home");
              return done();
            });
          });
          return it("should set location to \"Home\" by %LOC", function(done) {
            var parms;
            ssid = "UNKNOWN";
            cellid = "%CELLID";
            loc = phoneLOC("Office");
            locn = "%LOCN";
            parms = "?serial=" + serial + "&ssid=" + ssid + "&cellid=" + cellid + "&locn=" + locn + "&loc=" + loc;
            return phoneRequest(path + parms, function(body) {
              assert(JSON.parse(body).success);
              phoneStatus("phone");
              phoneCheckLocation("phone", "Home");
              return done();
            });
          });
        });
      });
    });
  });

}).call(this);

//# sourceMappingURL=phone-test.js.map
